@@ -302,14 +302,13 @@ def invoke_rest_api_integration(api_id, stage, integration, method, path, invoca
     relative_path, query_string_params = extract_query_string_params(path=invocation_path)
     integration_type_orig = integration.get('type') or integration.get('integrationType') or ''
     integration_type = integration_type_orig.upper()
-    uri = integration.get('uri') or integration.get('integrationUri')
+    uri = integration.get('uri') or integration.get('integrationUri') or ''
 
     if (uri.startswith('arn:aws:apigateway:') and ':lambda:path' in uri) or uri.startswith('arn:aws:lambda'):
         if integration_type in ['AWS', 'AWS_PROXY']:
             func_arn = uri
             if ':lambda:path' in uri:
                 func_arn = uri.split(':lambda:path')[1].split('functions/')[1].split('/invocations')[0]
-            data_str = json.dumps(data) if isinstance(data, (dict, list)) else to_str(data)
 
             try:
                 path_params = extract_path_params(path=relative_path, extracted_path=resource_path)
@@ -317,8 +316,13 @@ def invoke_rest_api_integration(api_id, stage, integration, method, path, invoca
                 path_params = {}
 
             # apply custom request template
-            data_str = apply_template(integration, 'request', data_str, path_params=path_params,
-                query_params=query_string_params, headers=headers)
+            data_str = data
+            try:
+                data_str = json.dumps(data) if isinstance(data, (dict, list)) else to_str(data)
+                data_str = apply_template(integration, 'request', data_str, path_params=path_params,
+                    query_params=query_string_params, headers=headers)
+            except Exception:
+                pass
 
             # Sample request context:
             # https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-create-api-as-simple-proxy-for-lambda.html#api-gateway-create-api-as-simple-proxy-for-lambda-test
@@ -388,6 +392,7 @@ def invoke_rest_api_integration(api_id, stage, integration, method, path, invoca
             if uri.endswith('states:action/StartExecution'):
                 action = 'StartExecution'
             decoded_data = data.decode()
+            payload = {}
             if 'stateMachineArn' in decoded_data and 'input' in decoded_data:
                 payload = json.loads(decoded_data)
             elif APPLICATION_JSON in integration.get('requestTemplates', {}):
