@@ -17,12 +17,12 @@ from flask import Flask, Response, jsonify, request
 from six.moves import cStringIO as StringIO
 from six.moves.urllib.parse import urlparse
 from localstack import config
-from localstack.constants import TEST_AWS_ACCOUNT_ID
+from localstack.constants import APPLICATION_JSON, TEST_AWS_ACCOUNT_ID
 from localstack.utils.aws import aws_stack, aws_responses
 from localstack.utils.common import (
     to_str, to_bytes, load_file, save_file, TMP_FILES, ensure_readable, short_uid, long_uid, json_safe,
     mkdir, unzip, is_zip_file, run, run_safe, first_char_to_lower, run_for_max_seconds, parse_request_data,
-    timestamp_millis, now_utc, safe_requests, FuncThread, isoformat_milliseconds, synchronized)
+    timestamp_millis, timestamp, now_utc, safe_requests, FuncThread, isoformat_milliseconds, synchronized)
 from localstack.services.awslambda import lambda_executors
 from localstack.services.generic_proxy import RegionBackend
 from localstack.services.awslambda.lambda_utils import (
@@ -123,6 +123,7 @@ class ClientError(Exception):
 
 
 class LambdaContext(object):
+    DEFAULT_MEMORY_LIMIT = 1536
 
     def __init__(self, func_details, qualifier=None, context=None):
         self.function_name = func_details.name()
@@ -132,6 +133,10 @@ class LambdaContext(object):
         if qualifier:
             self.invoked_function_arn += ':' + qualifier
         self.cognito_identity = context.get('identity')
+        self.aws_request_id = str(uuid.uuid4())
+        self.memory_limit_in_mb = func_details.memory_size or self.DEFAULT_MEMORY_LIMIT
+        self.log_group_name = '/aws/lambda/%s' % self.function_name
+        self.log_stream_name = '%s/[1]%s' % (timestamp(format='%Y/%m/%d'), short_uid())
 
     def get_remaining_time_in_millis(self):
         # TODO implement!
@@ -395,7 +400,11 @@ def start_lambda_sqs_listener():
 
             sqs_client = aws_stack.connect_to_service('sqs')
             entries = [{'Id': r['receiptHandle'], 'ReceiptHandle': r['receiptHandle']} for r in records]
-            sqs_client.delete_message_batch(QueueUrl=queue_url, Entries=entries)
+            try:
+                sqs_client.delete_message_batch(QueueUrl=queue_url, Entries=entries)
+            except Exception as e:
+                LOG.info('Unable to delete Lambda events from SQS queue ' +
+                    '(please check SQS visibility timeout settings): %s - %s' % (entries, e))
 
         records = []
         for msg in messages:
@@ -417,7 +426,11 @@ def start_lambda_sqs_listener():
         event = {'Records': records}
 
         # TODO implement retries, based on "RedrivePolicy.maxReceiveCount" in the queue settings
-        run_lambda(func_arn=lambda_arn, event=event, context={}, asynchronous=True, callback=delete_messages)
+        res = run_lambda(func_arn=lambda_arn, event=event, context={},
+            asynchronous=True, callback=delete_messages)
+        if isinstance(res, lambda_executors.InvocationResult) and getattr(res.result, 'status_code', 0) >= 400:
+            return False
+        return True
 
     def listener_loop(*args):
         while True:
@@ -430,6 +443,8 @@ def start_lambda_sqs_listener():
                     SQS_LISTENER_THREAD.pop('_thread_')
                     return
 
+                unprocessed_messages = {}
+
                 sqs_client = aws_stack.connect_to_service('sqs')
                 for source in sources:
                     queue_arn = source['EventSourceArn']
@@ -439,16 +454,21 @@ def start_lambda_sqs_listener():
                     try:
                         region_name = queue_arn.split(':')[3]
                         queue_url = aws_stack.sqs_queue_url_for_arn(queue_arn)
-                        result = sqs_client.receive_message(
-                            QueueUrl=queue_url,
-                            MessageAttributeNames=['All'],
-                            MaxNumberOfMessages=batch_size
-                        )
-                        messages = result.get('Messages')
+                        messages = unprocessed_messages.pop(queue_arn, None)
                         if not messages:
-                            continue
+                            result = sqs_client.receive_message(
+                                QueueUrl=queue_url,
+                                MessageAttributeNames=['All'],
+                                MaxNumberOfMessages=batch_size
+                            )
+                            messages = result.get('Messages')
+                            if not messages:
+                                continue
 
-                        send_event_to_lambda(queue_arn, queue_url, lambda_arn, messages, region=region_name)
+                        LOG.debug('Sending event from event source %s to Lambda %s' % (queue_arn, lambda_arn))
+                        res = send_event_to_lambda(queue_arn, queue_url, lambda_arn, messages, region=region_name)
+                        if not res:
+                            unprocessed_messages[queue_arn] = messages
 
                     except Exception as e:
                         LOG.debug('Unable to poll SQS messages for queue %s: %s' % (queue_arn, e))
@@ -568,6 +588,7 @@ def run_lambda(func_arn, event, context={}, version=None,
         func_arn = aws_stack.fix_arn(func_arn)
         func_details = region.lambdas.get(func_arn)
         if not func_details:
+            LOG.debug('Unable to find details for Lambda %s in region %s' % (func_arn, region_name))
             result = not_found_error(msg='The resource specified in the request does not exist.')
             return lambda_executors.InvocationResult(result)
 
@@ -650,7 +671,7 @@ def get_handler_function_from_name(handler_name, runtime=LAMBDA_DEFAULT_RUNTIME)
 
 
 def error_response(msg, code=500, error_type='InternalFailure'):
-    LOG.info(msg)
+    LOG.debug(msg)
     return aws_responses.flask_error_response_json(msg, code=code, error_type=error_type)
 
 
@@ -699,7 +720,8 @@ def get_java_handler(zip_file_content, main_file, func_details=None):
             return result
         return execute
     raise ClientError(error_response(
-        'Unable to extract Java Lambda handler - file is not a valid zip/jar file', 400, error_type='ValidationError'))
+        'Unable to extract Java Lambda handler - file is not a valid zip/jar file (%s, %s bytes)' %
+        (main_file, len(zip_file_content or '')), 400, error_type='ValidationError'))
 
 
 def set_archive_code(code, lambda_name, zip_file_content=None):
@@ -891,7 +913,7 @@ def format_func_details(func_details, version=None, always_add_version=False):
         'State': 'Active',
         'LastUpdateStatus': 'Successful',
         'PackageType': func_details.package_type,
-        'ImageConfig': func_details.image_config
+        'ImageConfig': getattr(func_details, 'image_config', None)
     }
     if func_details.dead_letter_config:
         result['DeadLetterConfig'] = func_details.dead_letter_config
@@ -940,14 +962,17 @@ def forward_to_fallback_url(func_arn, data):
         item = {
             'id': {'S': short_uid()},
             'timestamp': {'N': str(now_utc())},
-            'payload': {'S': str(data)},
+            'payload': {'S': data},
             'function_name': {'S': lambda_name}
         }
         aws_stack.create_dynamodb_table(table_name, partition_key='id')
         dynamodb.put_item(TableName=table_name, Item=item)
         return ''
     if re.match(r'^https?://.+', config.LAMBDA_FALLBACK_URL):
-        headers = {'lambda-function-name': lambda_name}
+        headers = {
+            'lambda-function-name': lambda_name,
+            'Content-Type': APPLICATION_JSON
+        }
         response = safe_requests.post(config.LAMBDA_FALLBACK_URL, data, headers=headers)
         content = response.content
         try:
@@ -1373,8 +1398,8 @@ def invoke_function(function):
         qualifier = request.args.get('Qualifier')
     data = request.get_data()
     if data:
-        data = to_str(data)
         try:
+            data = to_str(data)
             data = json.loads(data)
         except Exception:
             try:
@@ -1448,9 +1473,12 @@ def invoke_function(function):
         not_found = not_found_error('{0}:{1}'.format(arn, qualifier))
 
     if not_found:
-        forward_result = forward_to_fallback_url(arn, data)
-        if forward_result is not None:
-            return _create_response(forward_result)
+        try:
+            forward_result = forward_to_fallback_url(arn, json.dumps(data))
+            if forward_result is not None:
+                return _create_response(forward_result)
+        except Exception as e:
+            LOG.debug('Unable to forward Lambda invocation to fallback URL: "%s" - %s' % (data, e))
         return not_found
 
     if invocation_type == 'RequestResponse':
