@@ -1,7 +1,6 @@
 import re
 import json
 import logging
-
 from moto.ec2.utils import generate_route_id
 from moto.s3.models import FakeBucket
 from moto.sqs.models import Queue as MotoQueue
@@ -10,10 +9,11 @@ from moto.core.models import CloudFormationModel
 from moto.cloudformation.exceptions import UnformattedGetAttTemplateException
 from localstack.constants import AWS_REGION_US_EAST_1, LOCALHOST
 from localstack.utils.aws import aws_stack
-from localstack.utils.common import camel_to_snake_case, select_attributes
+from localstack.utils.common import camel_to_snake_case, select_attributes, canonical_json, md5
 from localstack.services.cloudformation.deployment_utils import (
     PLACEHOLDER_RESOURCE_NAME, remove_none_values, params_list_to_dict, lambda_keys_to_lower,
-    merge_parameters, params_dict_to_list, select_parameters, params_select_attributes)
+    merge_parameters, params_dict_to_list, select_parameters, params_select_attributes,
+    lambda_select_params)
 
 LOG = logging.getLogger(__name__)
 
@@ -76,6 +76,8 @@ class GenericBaseModel(CloudFormationModel):
     # TODO: change the signature to pass in a Stack instance (instead of stack_name and resources)
     def update_resource(self, new_resource, stack_name, resources):
         """ Update the deployment of this resource, using the updated properties (implemented by subclasses). """
+        # TODO: evaluate if we can add a generic implementation here, using "update" parameters from
+        # get_deploy_templates() responses, and based on checking whether resource attributes have changed
         pass
 
     @classmethod
@@ -728,7 +730,7 @@ class GatewayDeployment(GenericBaseModel):
         return 'AWS::ApiGateway::Deployment'
 
     def fetch_state(self, stack_name, resources):
-        api_id = self.props['RestApiId'] or self.resource_id
+        api_id = self.props.get('RestApiId') or self.resource_id
         api_id = self.resolve_refs_recursively(stack_name, api_id, resources)
         if not api_id:
             return None
@@ -1045,6 +1047,10 @@ class S3BucketPolicy(GenericBaseModel):
     def cloudformation_type():
         return 'AWS::S3::BucketPolicy'
 
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        policy = self.props.get('Policy')
+        return policy and md5(canonical_json(json.loads(policy)))
+
     def fetch_state(self, stack_name, resources):
         bucket_name = self.props.get('Bucket') or self.resource_id
         bucket_name = self.resolve_refs_recursively(stack_name, bucket_name, resources)
@@ -1257,12 +1263,98 @@ class SecretsManagerSecret(GenericBaseModel):
     def cloudformation_type():
         return 'AWS::SecretsManager::Secret'
 
+    def get_physical_resource_id(self, attribute, **kwargs):
+        props = self.props
+        result = props.get('Arn') or aws_stack.secretsmanager_secret_arn(props['Name'])
+        return result
+
+    def get_cfn_attribute(self, attribute_name):
+        if attribute_name in (REF_ARN_ATTRS + REF_ID_ATTRS):
+            return self.get_physical_resource_id(attribute_name)
+        return super(SecretsManagerSecret, self).get_cfn_attribute(attribute_name)
+
     def fetch_state(self, stack_name, resources):
         secret_name = self.props.get('Name') or self.resource_id
         secret_name = self.resolve_refs_recursively(stack_name, secret_name, resources)
         result = aws_stack.connect_to_service('secretsmanager').describe_secret(SecretId=secret_name)
         return result
 
+    @staticmethod
+    def get_deploy_templates():
+        return {
+            'create': {
+                'function': 'create_secret',
+                'parameters': lambda_select_params('Name', 'Description', 'KmsKeyId', 'SecretString', 'Tags')
+            },
+            'delete': {
+                'function': 'delete_secret',
+                'parameters': {
+                    'SecretId': 'Name'
+                }
+            }
+        }
+
+
+class SecretsManagerSecretTargetAttachment(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::SecretsManager::SecretTargetAttachment'
+
+    def get_physical_resource_id(self, attribute, **kwargs):
+        return aws_stack.secretsmanager_secret_arn(self.props.get('SecretId'))
+
+    def fetch_state(self, stack_name, resources):
+        # TODO implement?
+        return {'state': 'dummy'}
+
+
+class SecretsManagerRotationSchedule(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::SecretsManager::RotationSchedule'
+
+    def get_physical_resource_id(self, attribute, **kwargs):
+        return aws_stack.secretsmanager_secret_arn(self.props.get('SecretId'))
+
+    def fetch_state(self, stack_name, resources):
+        # TODO implement?
+        return {'state': 'dummy'}
+
+
+class SecretsManagerResourcePolicy(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::SecretsManager::ResourcePolicy'
+
+    def get_physical_resource_id(self, attribute, **kwargs):
+        return aws_stack.secretsmanager_secret_arn(self.props.get('SecretId'))
+
+    def fetch_state(self, stack_name, resources):
+        secret_id = self.resolve_refs_recursively(stack_name, self.props.get('SecretId'), resources)
+        result = aws_stack.connect_to_service('secretsmanager').get_resource_policy(SecretId=secret_id)
+        return result
+
+    @staticmethod
+    def get_deploy_templates():
+        def create_params(params, **kwargs):
+            return {
+                'SecretId': params['SecretId'].split(':')[-1],
+                'ResourcePolicy': json.dumps(params['ResourcePolicy']),
+                'BlockPublicPolicy': params.get('BlockPublicPolicy')
+            }
+        return {
+            'create': {
+                'function': 'put_resource_policy',
+                'parameters': create_params
+            },
+            'delete': {
+                'function': 'delete_resource_policy',
+                'parameters': {
+                    'SecretId': 'SecretId'
+                }
+            }
+        }
+
 
 class KMSKey(GenericBaseModel):
     @staticmethod
@@ -1270,10 +1362,85 @@ class KMSKey(GenericBaseModel):
         return 'AWS::KMS::Key'
 
     def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('kms')
         physical_res_id = self.physical_resource_id
+        props = self.props
+        res_tags = props.get('Tags', [])
         if not physical_res_id:
-            return None
-        return aws_stack.connect_to_service('kms').describe_key(KeyId=physical_res_id)
+            # TODO: find a more efficient approach for this?
+            for key in client.list_keys()['Keys']:
+                details = client.describe_key(KeyId=key['KeyId'])['KeyMetadata']
+                tags = client.list_resource_tags(KeyId=key['KeyId']).get('Tags', [])
+                tags = [{'Key': tag['TagKey'], 'Value': tag['TagValue']} for tag in tags]
+                if (tags == res_tags and details.get('Description') == props.get('Description') and
+                        props.get('KeyUsage') in [None, details.get('KeyUsage')]):
+                    physical_res_id = key['KeyId']
+                    # TODO should this be removed from here? It seems that somewhere along the execution
+                    # chain the 'PhysicalResourceId' gets overwritten with None, hence setting it here
+                    self.resource_json['PhysicalResourceId'] = physical_res_id
+                    break
+        if not physical_res_id:
+            return
+        return client.describe_key(KeyId=physical_res_id)
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        if attribute in REF_ID_ATTRS:
+            return self.physical_resource_id
+        return self.physical_resource_id and aws_stack.kms_key_arn(self.physical_resource_id)
+
+    @staticmethod
+    def get_deploy_templates():
+        def create_params(params, **kwargs):
+            return {
+                'Policy': params.get('KeyPolicy'),
+                'Tags': [{'TagKey': tag['Key'], 'TagValue': tag['Value']} for tag in params.get('Tags', [])]
+            }
+        return {
+            'create': {
+                'function': 'create_key',
+                'parameters': create_params
+            },
+            'delete': {
+                # TODO Key needs to be deleted in KMS backend
+                'function': 'schedule_key_deletion',
+                'parameters': {
+                    'KeyId': 'PhysicalResourceId'
+                }
+            }
+        }
+
+
+class KMSAlias(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::KMS::Alias'
+
+    def fetch_state(self, stack_name, resources):
+        kms = aws_stack.connect_to_service('kms')
+        aliases = kms.list_aliases()['Aliases']
+        for alias in aliases:
+            if alias['AliasName'] == self.props.get('AliasName'):
+                return alias
+
+        return None
+
+    @staticmethod
+    def get_deploy_templates():
+        return {
+            'create': {
+                'function': 'create_alias',
+                'parameters': {
+                    'AliasName': 'AliasName',
+                    'TargetKeyId': 'TargetKeyId'
+                }
+            },
+            'delete': {
+                'function': 'delete_alias',
+                'parameters': {
+                    'AliasName': 'AliasName'
+                }
+            },
+        }
 
 
 class EC2Instance(GenericBaseModel):
@@ -1308,7 +1475,6 @@ class EC2Instance(GenericBaseModel):
                 instance_id
             ]
         )
-
         return resp['Reservations'][0]['Instances'][0]
 
 
@@ -1361,8 +1527,8 @@ class EC2Subnet(GenericBaseModel):
         return 'AWS::EC2::Subnet'
 
     def fetch_state(self, stack_name, resources):
-        props = self.props
         client = aws_stack.connect_to_service('ec2')
+        props = self.props
         filters = [
             {'Name': 'cidr-block', 'Values': [props['CidrBlock']]},
             {'Name': 'vpc-id', 'Values': [props['VpcId']]}
@@ -1370,6 +1536,9 @@ class EC2Subnet(GenericBaseModel):
         subnets = client.describe_subnets(Filters=filters)['Subnets']
         return (subnets or [None])[0]
 
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        return self.props.get('SubnetId')
+
     @staticmethod
     def get_deploy_templates():
         return {
@@ -1392,9 +1561,6 @@ class EC2Subnet(GenericBaseModel):
             }
         }
 
-    def get_physical_resource_id(self, attribute=None, **kwargs):
-        return self.props.get('SubnetId')
-
 
 class EC2VPC(GenericBaseModel):
     @staticmethod
@@ -1431,6 +1597,44 @@ class EC2VPC(GenericBaseModel):
         return self.physical_resource_id or self.props.get('VpcId')
 
 
+class EC2NatGateway(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::EC2::NatGateway'
+
+    def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('ec2')
+        props = self.props
+        subnet_id = self.resolve_refs_recursively(stack_name, props.get('SubnetId'), resources)
+        assoc_id = self.resolve_refs_recursively(stack_name, props.get('AllocationId'), resources)
+        result = client.describe_nat_gateways(Filters=[{'Name': 'subnet-id', 'Values': [subnet_id]}])
+        result = result['NatGateways']
+        result = [gw for gw in result if assoc_id in [ga['AllocationId'] for ga in gw['NatGatewayAddresses']]]
+        return (result or [None])[0]
+
+    @staticmethod
+    def get_deploy_templates():
+        return {
+            'create': {
+                'function': 'create_nat_gateway',
+                'parameters': {
+                    'SubnetId': 'SubnetId',
+                    'AllocationId': 'AllocationId'
+                    # TODO: add TagSpecifications
+                }
+            },
+            'delete': {
+                'function': 'delete_nat_gateway',
+                'parameters': {
+                    'NatGatewayId': 'PhysicalResourceId'
+                }
+            }
+        }
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        return self.physical_resource_id or self.props.get('NatGatewayId')
+
+
 class InstanceProfile(GenericBaseModel):
     @staticmethod
     def cloudformation_type():
@@ -1514,10 +1718,11 @@ class EC2Route(GenericBaseModel):
         return 'AWS::EC2::Route'
 
     def get_physical_resource_id(self, attribute=None, **kwargs):
+        props = self.props
         return generate_route_id(
-            self.props.get('RouteTableId'),
-            self.props.get('DestinationCidrBlock'),
-            self.props.get('DestinationIpv6CidrBlock')
+            props.get('RouteTableId'),
+            props.get('DestinationCidrBlock'),
+            props.get('DestinationIpv6CidrBlock')
         )
 
     @staticmethod
@@ -1540,3 +1745,104 @@ class EC2Route(GenericBaseModel):
                 }
             }
         }
+
+
+class EC2InternetGateway(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::EC2::InternetGateway'
+
+    def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('ec2')
+        gateways = client.describe_internet_gateways()['InternetGateways']
+        tags = self.props.get('Tags')
+        gateway = [g for g in gateways if g.get('Tags') == tags]
+        return (gateway or [None])[0]
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        return self.props.get('InternetGatewayId')
+
+    @staticmethod
+    def get_deploy_templates():
+        def _create_params(params, **kwargs):
+            return {'TagSpecifications': [{'ResourceType': 'internet-gateway', 'Tags': params.get('Tags', [])}]}
+        return {
+            'create': {
+                'function': 'create_internet_gateway',
+                'parameters': _create_params
+            }
+        }
+
+
+class EC2SubnetRouteTableAssociation(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::EC2::SubnetRouteTableAssociation'
+
+    def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('ec2')
+        table_id = self.resolve_refs_recursively(stack_name, self.props.get('RouteTableId'), resources)
+        gw_id = self.resolve_refs_recursively(stack_name, self.props.get('GatewayId'), resources)
+        route_tables = client.describe_route_tables()['RouteTables']
+        route_table = ([t for t in route_tables if t['RouteTableId'] == table_id] or [None])[0]
+        if route_table:
+            associations = route_table.get('Associations', [])
+            association = [a for a in associations if a.get('GatewayId') == gw_id]
+            return (association or [None])[0]
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        return self.props.get('RouteTableAssociationId')
+
+    @staticmethod
+    def get_deploy_templates():
+        return {
+            'create': {
+                'function': 'associate_route_table',
+                'parameters': {
+                    'GatewayId': 'GatewayId',
+                    'RouteTableId': 'RouteTableId',
+                    'SubnetId': 'SubnetId'
+                }
+            }
+        }
+
+
+class EC2VPCGatewayAttachment(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::EC2::VPCGatewayAttachment'
+
+    def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('ec2')
+        igw_id = self.resolve_refs_recursively(stack_name, self.props.get('InternetGatewayId'), resources)
+        vpngw_id = self.resolve_refs_recursively(stack_name, self.props.get('VpnGatewayId'), resources)
+        gateways = []
+        if igw_id:
+            gateways = client.describe_internet_gateways()['InternetGateways']
+            gateways = [g for g in gateways if g['InternetGatewayId'] == igw_id]
+        elif vpngw_id:
+            gateways = client.describe_vpn_gateways()['VpnGateways']
+            gateways = [g for g in gateways if g['VpnGatewayId'] == vpngw_id]
+        return (gateways or [None])[0]
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        return self.props.get('RouteTableAssociationId')
+
+    @staticmethod
+    def get_deploy_templates():
+        def _attach_gateway(resource_id, resources, *args, **kwargs):
+            client = aws_stack.connect_to_service('ec2')
+            resource = resources[resource_id]
+            resource_props = resource.get('Properties')
+            igw_id = resource_props.get('InternetGatewayId')
+            vpngw_id = resource_props.get('VpnGatewayId')
+            vpc_id = resource_props.get('VpcId')
+            if igw_id:
+                client.attach_internet_gateway(VpcId=vpc_id, InternetGatewayId=igw_id)
+            elif vpngw_id:
+                client.attach_vpn_gateway(VpcId=vpc_id, VpnGatewayId=vpngw_id)
+        return {
+            'create': {
+                'function': _attach_gateway
+            }
+        }
