@@ -13,8 +13,8 @@ from localstack.services import plugins
 from localstack.dashboard import infra as dashboard_infra
 from localstack.utils.aws import aws_stack
 from localstack.constants import (
-    HEADER_LOCALSTACK_TARGET, HEADER_LOCALSTACK_EDGE_URL, LOCALSTACK_ROOT_FOLDER,
-    PATH_USER_REQUEST, LOCALHOST, LOCALHOST_IP)
+    HEADER_LOCALSTACK_TARGET, HEADER_LOCALSTACK_EDGE_URL, HEADER_LOCALSTACK_REQUEST_URL,
+    LOCALSTACK_ROOT_FOLDER, PATH_USER_REQUEST, LOCALHOST, LOCALHOST_IP)
 from localstack.utils.common import (
     empty_context_manager, run, is_root, TMP_THREADS, to_bytes, truncate, to_str,
     get_service_protocol, in_docker, safe_requests as requests, parse_request_data)
@@ -57,11 +57,17 @@ class ProxyListenerEdge(ProxyListener):
         if auth_header and not headers.get('authorization'):
             headers['authorization'] = auth_header
         host = headers.get('host', '')
-        headers[HEADER_LOCALSTACK_EDGE_URL] = 'https://%s' % host
+        orig_req_url = headers.pop(HEADER_LOCALSTACK_REQUEST_URL, '')
+        headers[HEADER_LOCALSTACK_EDGE_URL] = (
+            re.sub(r'^([^:]+://[^/]+).*', r'\1', orig_req_url) or 'http://%s' % host)
 
         # extract API details
         api, port, path, host = get_api_from_headers(headers, method=method, path=path, data=data)
 
+        if api and config.LS_LOG:
+            # print request trace for debugging, if enabled
+            LOG.debug('IN(%s): "%s %s" - headers: %s - data: %s' % (api, method, path, dict(headers), data))
+
         set_default_region_in_headers(headers)
 
         if port and int(port) < 0:
@@ -72,6 +78,9 @@ class ProxyListenerEdge(ProxyListener):
 
         if not port:
             if method == 'OPTIONS':
+                if api and config.LS_LOG:
+                    # print request trace for debugging, if enabled
+                    LOG.debug('OUT(%s): "%s %s" - status: %s' % (api, method, path, 200))
                 return 200
 
             if api in ['', None, '_unknown_']:
@@ -108,6 +117,14 @@ class ProxyListenerEdge(ProxyListener):
             return do_forward_request(api, method, path, data, headers, port=port)
 
     def return_response(self, method, path, data, headers, response, request_handler=None):
+
+        if config.LS_LOG:
+            # print response trace for debugging, if enabled
+            api, port, path, host = get_api_from_headers(headers, method=method, path=path, data=data)
+            if api and api != '_unknown_':
+                LOG.debug('OUT(%s): "%s %s" - status: %s - response headers: %s - response: %s' %
+                    (api, method, path, response.status_code, dict(response.headers), response.content))
+
         if headers.get('Accept-Encoding') == 'gzip' and response._content:
             response._content = gzip.compress(to_bytes(response._content))
             response.headers['Content-Length'] = str(len(response._content))
@@ -170,7 +187,6 @@ def get_auth_string(method, path, headers, data=None):
        &X-Amz-Credential=test%2F20210313%2Fus-east-1%2Fsqs%2Faws4_request\
        &X-Amz-Date=20210313T011059Z&X-Amz-Expires=86400000&X-Amz-SignedHeaders=host\
        &X-Amz-Signature=2c652c7bc9a3b75579db3d987d1e6dd056f0ac776c1e1d4ec91e2ce84e5ad3ae
-
     """
 
     auth_header = headers.get('authorization', '')
@@ -298,7 +314,8 @@ def terminate_all_processes_in_docker():
 def serve_resource_graph(data):
     data = json.loads(to_str(data or '{}'))
     env = Environment.from_string(data.get('awsEnvironment'))
-    graph = dashboard_infra.get_graph(name_filter=data.get('nameFilter') or '.*', env=env, region=data.get('awsRegion'))
+    graph = dashboard_infra.get_graph(name_filter=data.get('nameFilter') or '.*',
+        env=env, region=data.get('awsRegion'))
     return graph
 
 
@@ -384,6 +401,10 @@ def get_api_from_custom_rules(method, path, data, headers):
     if auth_header.startswith('AWS '):
         return 's3', config.PORT_S3
 
+    # certain EC2 requests from Java SDK contain no Auth headers (issue #3805)
+    if b'Version=2016-11-15' in data_bytes:
+        return 'ec2', config.PORT_EC2
+
 
 def get_service_port_for_account(service, headers):
     # assume we're only using a single account, hence return the static port mapping from config.py
