@@ -56,10 +56,10 @@ def filter_event_based_on_event_format(self, rule, event):
                 if not filter_event_with_content_base_parameter(value, event_value):
                     return False
 
-            elif isinstance(value, (str, int)):
+            elif isinstance(value, (str, dict)):
                 try:
-                    if isinstance(json.loads(value), dict) and \
-                       not filter_event(json.loads(value), event_value):
+                    value = json.loads(value) if isinstance(value, str) else value
+                    if isinstance(value, dict) and not filter_event(value, event_value):
                         return False
                 except json.decoder.JSONDecodeError:
                     return False
@@ -67,8 +67,8 @@ def filter_event_based_on_event_format(self, rule, event):
 
     rule_information = self.events_backend.describe_rule(rule)
     if rule_information.event_pattern:
-        event_pattern = json.loads(rule_information.event_pattern)
-        if not filter_event(event_pattern, event):
+        event_pattern = rule_information.event_pattern._filter
+        if event_pattern and not filter_event(event_pattern, event):
             return False
     return True
 
@@ -113,24 +113,6 @@ def apply_patches():
 
         return events_handler_delete_rule_orig(self)
 
-    # 2101 Events put-targets does not respond
-    def events_handler_put_targets(self):
-        rule_name = self._get_param('Rule')
-        targets = self._get_param('Targets')
-
-        if not rule_name:
-            return self.error('ValidationException', 'Parameter Rule is required.')
-
-        if not targets:
-            return self.error('ValidationException', 'Parameter Targets is required.')
-
-        if not self.events_backend.put_targets(rule_name, targets):
-            return self.error(
-                'ResourceNotFoundException', 'Rule ' + rule_name + ' does not exist.'
-            )
-
-        return json.dumps({'FailedEntryCount': 0, 'FailedEntries': []}), self.response_headers
-
     def events_handler_put_events(self):
         entries = self._get_param('Entries')
         events = list(
@@ -180,7 +162,6 @@ def apply_patches():
     rule_model._generate_arn = rule_model_generate_arn
     events_handler.put_rule = events_handler_put_rule
     events_handler.delete_rule = events_handler_delete_rule
-    events_handler.put_targets = events_handler_put_targets
     events_handler.put_events = events_handler_put_events
 
 
