@@ -5,14 +5,11 @@ from gzip import GzipFile
 from localstack import config
 from localstack.utils.aws import aws_stack
 from moto.awslambda import models as lambda_models
-from moto.logs import models as logs_models
-from moto.logs.models import LogStream
+from moto.logs import models as logs_models, responses as logs_responses
 from moto.core.utils import unix_time_millis
+from moto.logs.models import LogStream
+from moto.logs.exceptions import ResourceNotFoundException, InvalidParameterException
 from localstack.services.infra import start_moto_server
-from moto.logs.exceptions import (
-    ResourceNotFoundException,
-    InvalidParameterException,
-)
 
 
 def patch_lambda():
@@ -146,6 +143,38 @@ def patch_lambda():
     for logs_backend in logs_models.logs_backends.values():
         logs_backend.put_subscription_filter = patch_put_subscription_filter(logs_backend)
 
+    def put_metric_filter(self):
+        data = dict(self.request_params)
+        metric_filters = self.logs_backend.metric_filters = getattr(self.logs_backend, 'metric_filters', [])
+        metric_filters.append(data)
+        return json.dumps({})
+
+    if not hasattr(logs_responses.LogsResponse, 'put_metric_filter'):
+        logs_responses.LogsResponse.put_metric_filter = put_metric_filter
+
+    def describe_metric_filters(self):
+        log_group_name = self._get_param('logGroupName')
+        name_prefix = self._get_param('filterNamePrefix') or ''
+        metric_filters = self.logs_backend.metric_filters = getattr(self.logs_backend, 'metric_filters', [])
+        metric_filters = [mf for mf in metric_filters if log_group_name in (None, mf['logGroupName'])]
+        metric_filters = [mf for mf in metric_filters if mf['filterName'].startswith(name_prefix)]
+        result = {'metricFilters': metric_filters}
+        return json.dumps(result)
+
+    if not hasattr(logs_responses.LogsResponse, 'describe_metric_filters'):
+        logs_responses.LogsResponse.describe_metric_filters = describe_metric_filters
+
+    def delete_metric_filter(self):
+        log_group_name = self._get_param('logGroupName')
+        filter_name = self._get_param('filterName')
+        metric_filters = self.logs_backend.metric_filters = getattr(self.logs_backend, 'metric_filters', [])
+        self.logs_backend.metric_filters = [mf for mf in metric_filters
+            if mf['filterName'] != filter_name or mf['logGroupName'] != log_group_name]
+        return json.dumps({})
+
+    if not hasattr(logs_responses.LogsResponse, 'delete_metric_filter'):
+        logs_responses.LogsResponse.delete_metric_filter = delete_metric_filter
+
 
 def start_cloudwatch_logs(port=None, asynchronous=False, update_listener=None):
     port = port or config.PORT_LOGS
