@@ -14,6 +14,7 @@ import six
 import botocore.config
 from pytz import timezone
 from urllib.parse import parse_qs
+from moto.s3.models import s3_backend
 from botocore.client import ClientError
 from requests.models import Response, Request
 from six.moves.urllib import parse as urlparse
@@ -29,7 +30,6 @@ from localstack.utils.common import (
     short_uid, timestamp_millis, to_str, to_bytes, clone, md5, get_service_protocol, now_utc, is_base64
 )
 from localstack.utils.analytics import event_publisher
-from localstack.utils.http_utils import uses_chunked_encoding
 from localstack.utils.persistence import PersistingProxyListener
 from localstack.utils.aws.aws_responses import requests_response
 from localstack.services.cloudformation.service_models import S3Bucket
@@ -41,25 +41,22 @@ STREAMING_HMAC_PAYLOAD = 'STREAMING-AWS4-HMAC-SHA256-PAYLOAD'
 PORT_S3_BACKEND = None
 
 # mappings for S3 bucket notifications
-S3_NOTIFICATIONS = {}
+S3_NOTIFICATIONS = s3_backend.S3_NOTIFICATIONS = getattr(s3_backend, 'S3_NOTIFICATIONS', {})
 
 # mappings for bucket CORS settings
-BUCKET_CORS = {}
+BUCKET_CORS = s3_backend.BUCKET_CORS = getattr(s3_backend, 'BUCKET_CORS', {})
 
 # maps bucket name to lifecycle settings
-BUCKET_LIFECYCLE = {}
+BUCKET_LIFECYCLE = s3_backend.BUCKET_LIFECYCLE = getattr(s3_backend, 'BUCKET_LIFECYCLE', {})
 
 # maps bucket name to replication settings
-BUCKET_REPLICATIONS = {}
-
-# maps bucket name to encryption settings
-BUCKET_ENCRYPTIONS = {}
+BUCKET_REPLICATIONS = s3_backend.BUCKET_REPLICATIONS = getattr(s3_backend, 'BUCKET_REPLICATIONS', {})
 
 # maps bucket name to object lock settings
-OBJECT_LOCK_CONFIGS = {}
+OBJECT_LOCK_CONFIGS = s3_backend.OBJECT_LOCK_CONFIGS = getattr(s3_backend, 'OBJECT_LOCK_CONFIGS', {})
 
 # map to store the s3 expiry dates
-OBJECT_EXPIRY = {}
+OBJECT_EXPIRY = s3_backend.OBJECT_EXPIRY = getattr(s3_backend, 'OBJECT_EXPIRY', {})
 
 # set up logger
 LOGGER = logging.getLogger(__name__)
@@ -424,9 +421,6 @@ def add_response_metadata_headers(response):
         response.headers['content-language'] = 'en-US'
     if response.headers.get('cache-control') is None:
         response.headers['cache-control'] = 'no-cache'
-    if response.headers.get('content-encoding') is None:
-        if not uses_chunked_encoding(response):
-            response.headers['content-encoding'] = 'identity'
 
 
 def append_last_modified_headers(response, content=None):
@@ -618,7 +612,7 @@ def remove_xml_preamble(response):
 
 # --------------
 # HELPER METHODS
-#   for lifecycle/replication/encryption/...
+#   for lifecycle/replication/...
 # --------------
 
 def get_lifecycle(bucket_name):
@@ -662,26 +656,6 @@ def get_replication(bucket_name):
     return requests_response(body, status_code=status_code)
 
 
-def get_encryption(bucket_name):
-    bucket_name = normalize_bucket_name(bucket_name)
-    exists, code, body = is_bucket_available(bucket_name)
-    if not exists:
-        return requests_response(body, status_code=code)
-
-    encryption = BUCKET_ENCRYPTIONS.get(bucket_name)
-    status_code = 200
-    if not encryption:
-        encryption = {
-            'Error': {
-                'Code': 'ServerSideEncryptionConfigurationNotFoundError',
-                'Message': 'The server side encryption configuration was not found'
-            }
-        }
-        status_code = 404
-    body = xmltodict.unparse(encryption)
-    return requests_response(body, status_code=status_code)
-
-
 def get_object_lock(bucket_name):
     bucket_name = normalize_bucket_name(bucket_name)
     exists, code, body = is_bucket_available(bucket_name)
@@ -736,18 +710,6 @@ def set_replication(bucket_name, replication):
     return 200
 
 
-def set_encryption(bucket_name, encryption):
-    bucket_name = normalize_bucket_name(bucket_name)
-    exists, code, body = is_bucket_available(bucket_name)
-    if not exists:
-        return requests_response(body, status_code=code)
-
-    if isinstance(to_str(encryption), six.string_types):
-        encryption = xmltodict.parse(encryption)
-    BUCKET_ENCRYPTIONS[bucket_name] = encryption
-    return 200
-
-
 def set_object_lock(bucket_name, lock_config):
     bucket_name = normalize_bucket_name(bucket_name)
     exists, code, body = is_bucket_available(bucket_name)
@@ -1141,12 +1103,6 @@ class ProxyListenerS3(PersistingProxyListener):
             if method == 'PUT':
                 return set_replication(bucket_name, data)
 
-        if query == 'encryption' or 'encryption' in query_map:
-            if method == 'GET':
-                return get_encryption(bucket_name)
-            if method == 'PUT':
-                return set_encryption(bucket_name, data)
-
         if query == 'object-lock' or 'object-lock' in query_map:
             if method == 'GET':
                 return get_object_lock(bucket_name)
@@ -1246,8 +1202,10 @@ class ProxyListenerS3(PersistingProxyListener):
                 if aws_stack.get_region() == 'us-east-1':
                     response.headers['Location'] = '/{}'.format(bucket_name)
                 else:
-                    response.headers['Location'] = 'http://{}.{}:{}/'.format(
-                        bucket_name, constants.S3_VIRTUAL_HOSTNAME, config.EDGE_PORT)
+                    # Note: we need to set the correct protocol here
+                    protocol = headers.get(constants.HEADER_LOCALSTACK_EDGE_URL, '').split('://')[0] or 'http'
+                    response.headers['Location'] = '{}://{}.{}:{}/'.format(
+                        protocol, bucket_name, constants.S3_VIRTUAL_HOSTNAME, config.EDGE_PORT)
 
         if response is not None:
             reset_content_length = False
