@@ -20,9 +20,11 @@ LOGGER = logging.getLogger(__name__)
 REGION_REGEX = r'[a-z]{2}-[a-z]+-[0-9]{1,}'
 PORT_REGEX = r'(:[\d]{0,6})?'
 S3_STATIC_WEBSITE_HOST_REGEX = r'^([^.]+)\.s3-website\.localhost\.localstack\.cloud(:[\d]{0,6})?$'
-S3_VIRTUAL_HOSTNAME_REGEX = (r'^(http(s)?://)?([^\.]+)\.s3((-website)|(-external-1))?[\.-](dualstack\.)?'
-                             r'((localhost\.localstack\.cloud)|'
-                             r'(({}\.)?amazonaws\.com(.cn)?)){}$'.format(REGION_REGEX, PORT_REGEX))
+S3_VIRTUAL_HOSTNAME_REGEX = (r'^(http(s)?://)?((?!s3\.)[^\./]+)\.'
+                             r'(((s3(-website)?\.)?localhost\.localstack\.cloud)|({})|'
+                             r'(s3((-website)|(-external-1))?[\.-](dualstack\.)?'
+                             r'({}\.)?amazonaws\.com(.cn)?)){}$').format(
+                                 config.HOSTNAME_EXTERNAL, REGION_REGEX, PORT_REGEX)
 BUCKET_NAME_REGEX = (r'(?=^.{3,63}$)(?!^(\d+\.)+\d+$)' +
     r'(^(([a-z0-9]|[a-z0-9][a-z0-9\-]*[a-z0-9])\.)*([a-z0-9]|[a-z0-9][a-z0-9\-]*[a-z0-9])$)')
 
@@ -132,6 +134,10 @@ def get_forwarded_for_host(headers):
     return host
 
 
+def is_real_s3_url(url):
+    return re.match(r'.*s3(\-website)?\.([^\.]+\.)?amazonaws.com.*', url or '')
+
+
 def authenticate_presign_url(method, path, headers, data=None):
 
     url = '{}{}'.format(config.get_edge_url(), path)
