@@ -246,8 +246,7 @@ def connect_to_service(service_name, client=True, env=None, region_name=None, en
         # configure S3 path style addressing
         if service_name == 's3':
             if re.match(r'https?://localhost(:[0-9]+)?', endpoint_url):
-                endpoint_url = endpoint_url.replace('localhost', S3_VIRTUAL_HOSTNAME)
-                config.s3 = {'addressing_style': 'auto'}
+                endpoint_url = endpoint_url.replace('://localhost', '://%s' % S3_VIRTUAL_HOSTNAME)
         # To, prevent error "Connection pool is full, discarding connection ...",
         # set the environment variable MAX_POOL_CONNECTIONS. Default is 150.
         config.max_pool_connections = MAX_POOL_CONNECTIONS
@@ -415,9 +414,11 @@ def get_iam_role(resource, env=None):
     return 'role-%s' % resource
 
 
-def secretsmanager_secret_arn(secret_name, account_id=None, region_name=None):
+def secretsmanager_secret_arn(secret_id, account_id=None, region_name=None):
+    if ':' in (secret_id or ''):
+        return secret_id
     pattern = 'arn:aws:secretsmanager:%s:%s:secret:%s'
-    return _resource_arn(secret_name, pattern, account_id=account_id, region_name=region_name)
+    return _resource_arn(secret_id, pattern, account_id=account_id, region_name=region_name)
 
 
 def cloudformation_stack_arn(stack_name, stack_id=None, account_id=None, region_name=None):
@@ -536,6 +537,11 @@ def es_domain_arn(domain_name, account_id=None, region_name=None):
     return _resource_arn(domain_name, pattern, account_id=account_id, region_name=region_name)
 
 
+def kms_key_arn(key_id, account_id=None, region_name=None):
+    pattern = 'arn:aws:kms:%s:%s:key/%s'
+    return _resource_arn(key_id, pattern, account_id=account_id, region_name=region_name)
+
+
 def code_signing_arn(code_signing_id, account_id=None, region_name=None):
     pattern = 'arn:aws:lambda:%s:%s:code-signing-config:%s'
     return _resource_arn(code_signing_id, pattern, account_id=account_id, region_name=region_name)
@@ -943,8 +949,8 @@ def kinesis_get_latest_records(stream_name, shard_id, count=10, env=None):
     return result
 
 
-def get_stack_details(stack_name):
-    cloudformation = connect_to_service('cloudformation')
+def get_stack_details(stack_name, region_name=None):
+    cloudformation = connect_to_service('cloudformation', region_name=region_name)
     stacks = cloudformation.describe_stacks(StackName=stack_name)
     for stack in stacks['Stacks']:
         if stack['StackName'] == stack_name:
@@ -958,9 +964,9 @@ def deploy_cf_stack(stack_name, template_body):
     return await_stack_completion(stack_name)
 
 
-def await_stack_status(stack_name, expected_statuses, retries=20, sleep=2):
+def await_stack_status(stack_name, expected_statuses, retries=20, sleep=2, region_name=None):
     def check_stack():
-        stack = get_stack_details(stack_name)
+        stack = get_stack_details(stack_name, region_name=region_name)
         if stack['StackStatus'] not in expected_statuses:
             raise Exception('Status "%s" for stack "%s" not in expected list: %s' % (
                 stack['StackStatus'], stack_name, expected_statuses))
@@ -970,20 +976,6 @@ def await_stack_status(stack_name, expected_statuses, retries=20, sleep=2):
     return retry(check_stack, retries, sleep)
 
 
-def await_stack_completion(stack_name, retries=20, sleep=2, statuses=None):
+def await_stack_completion(stack_name, retries=20, sleep=2, statuses=None, region_name=None):
     statuses = statuses or ['CREATE_COMPLETE', 'UPDATE_COMPLETE']
-    return await_stack_status(stack_name, statuses, retries=retries, sleep=sleep)
-
-
-# TODO: move to aws_responses.py?
-def extract_tags(req_data):
-    tags = []
-    for i in range(1, 200):
-        k1 = 'Tags.member.%s.Key' % i
-        k2 = 'Tags.member.%s.Value' % i
-        key = req_data.get(k1)
-        value = req_data.get(k2, '')
-        if key is None:
-            break
-        tags.append({'Key': key, 'Value': value})
-    return tags
+    return await_stack_status(stack_name, statuses, retries=retries, sleep=sleep, region_name=region_name)
