@@ -3,7 +3,6 @@ import re
 import sys
 import json
 import time
-import yaml
 import select
 import pkgutil
 import logging
@@ -275,9 +274,9 @@ def setup_logging(log_level=None):
 
     # overriding the log level if LS_LOG has been set
     if config.LS_LOG:
-        LS_LOG = str(config.LS_LOG).upper()
-        LS_LOG = 'WARNING' if LS_LOG == 'WARN' else LS_LOG
-        log_level = getattr(logging, LS_LOG)
+        log_level = str(config.LS_LOG).upper()
+        log_level = 'WARNING' if log_level == 'WARN' else 'DEBUG' if log_level == 'TRACE' else log_level
+        log_level = getattr(logging, log_level)
         logging.getLogger('').setLevel(log_level)
         logging.getLogger('localstack').setLevel(log_level)
 
@@ -386,6 +385,7 @@ def validate_localstack_config(name):
         return False
 
     # validating docker-compose variable
+    import yaml  # keep import here to avoid issues in test Lambdas
     with open(compose_file_name) as file:
         compose_content = yaml.full_load(file)
     services_config = compose_content.get('services', {})
@@ -413,15 +413,14 @@ def validate_localstack_config(name):
 
     # docker-compose file validation cases
 
-    if (docker_env.get('LAMBDA_REMOTE_DOCKER') in constants.FALSE_STRINGS and
-            docker_env.get('HOST_TMP_FOLDER') in ['${TMPDIR}', None, '']):
-        warns.append('Make sure to properly set the "HOST_TMP_FOLDER" environment variable for the '
-                    'LocalStack container when using "LAMBDA_REMOTE_DOCKER=false"')
-
     if docker_env.get('PORT_WEB_UI') not in ['${PORT_WEB_UI- }', None, ''] and image_name == 'localstack/localstack':
         warns.append('"PORT_WEB_UI" Web UI is now deprecated, '
                     'and requires to use the "localstack/localstack-full" image.')
 
+    if not docker_env.get('HOST_TMP_FOLDER'):
+        warns.append('Please configure the "HOST_TMP_FOLDER" environment variable to point to the ' +
+            'absolute path of a temp folder on your host system (e.g., HOST_TMP_FOLDER=${TMPDIR})')
+
     if (main_container not in container_name) and not docker_env.get('MAIN_CONTAINER_NAME'):
         warns.append('Please use "container_name: %s" or add "MAIN_CONTAINER_NAME" in "environment".' %
             main_container)
