@@ -24,6 +24,7 @@ import six
 import shutil
 import requests
 import dns.resolver
+import platform
 from io import BytesIO
 from datetime import datetime, date
 from contextlib import closing
@@ -427,6 +428,7 @@ def md5(string):
 
 
 def select_attributes(object, attributes):
+    attributes = attributes if isinstance(attributes, list) else [attributes]
     return dict([(k, v) for k, v in object.items() if k in attributes])
 
 
@@ -468,6 +470,8 @@ def path_from_url(url):
 
 def is_port_open(port_or_url, http_path=None, expect_success=True, protocols=['tcp']):
     port = port_or_url
+    if is_number(port):
+        port = int(port)
     host = 'localhost'
     protocol = 'http'
     protocols = protocols if isinstance(protocols, list) else [protocols]
@@ -732,6 +736,8 @@ def chown_r(path, user):
 
 def chmod_r(path, mode):
     """ Recursive chmod """
+    if not os.path.exists(path):
+        return
     os.chmod(path, mode)
     for root, dirnames, filenames in os.walk(path):
         for dirname in dirnames:
@@ -842,11 +848,17 @@ def download(url, path, verify_ssl=True):
         s.close()
 
 
-def parse_request_data(method, path, data):
+def parse_request_data(method, path, data, headers={}):
     """ Extract request data either from query string (for GET) or request body (for POST). """
     result = {}
-    if method in ['POST', 'PUT', 'PATCH']:
-        result = parse_qs(to_str(data or ''))
+    headers = headers or {}
+    content_type = headers.get('Content-Type', '')
+    if method in ['POST', 'PUT', 'PATCH'] and (not content_type or 'form-' in content_type):
+        # content-type could be either "application/x-www-form-urlencoded" or "multipart/form-data"
+        try:
+            result = parse_qs(to_str(data or ''))
+        except Exception:
+            pass  # probably binary / JSON / non-URL encoded payload - ignore
     if not result:
         parsed_path = urlparse(path)
         result = parse_qs(parsed_path.query)
@@ -855,7 +867,11 @@ def parse_request_data(method, path, data):
 
 
 def first_char_to_lower(s):
-    return '%s%s' % (s[0].lower(), s[1:])
+    return s and '%s%s' % (s[0].lower(), s[1:])
+
+
+def first_char_to_upper(s):
+    return s and '%s%s' % (s[0].upper(), s[1:])
 
 
 def format_number(number, decimals=2):
@@ -878,6 +894,10 @@ def is_linux():
     return bootstrap.is_linux()
 
 
+def is_windows():
+    return platform.system().lower() == 'windows'
+
+
 def is_alpine():
     try:
         with MUTEX_CLEAN:
@@ -899,6 +919,8 @@ def get_arch():
         return 'alpine'
     if is_linux():
         return 'linux'
+    if is_windows():
+        return 'windows'
     raise Exception('Unable to determine system architecture')
 
 
@@ -1480,6 +1502,16 @@ def escape_html(string, quote=False):
     return html.escape(string, quote=quote)
 
 
+def get_all_subclasses(clazz):
+    """ Recursively get all subclasses of the given class. """
+    result = set()
+    subs = clazz.__subclasses__()
+    for sub in subs:
+        result.add(sub)
+        result.update(get_all_subclasses(sub))
+    return result
+
+
 def parallelize(func, list, size=None):
     if not size:
         size = len(list)
